use cdtypes::types::cdstring::DString;
use tool_helper::Error;

pub struct CDDesc {
    pub root: Directory
}

impl CDDesc {
    pub fn new() -> CDDesc {
        match Directory::new("root") {
            Ok(root)   => CDDesc{root},
            Err(error) => panic!("Creating root directory failed with: {}", error)
        }
    }
}

pub struct PVD {

}

pub struct Directory {
    name:       DirectoryName,
    properties: Properties,
    data:       Vec<Data>,
    dirs:       Vec<Directory>
}

impl Directory {
    pub fn new(dir_name: &str) -> Result<Directory, Error> {
        Ok(Directory{name: DirectoryName::from_str(dir_name)?, properties: Properties::default(), data: Vec::new(), dirs: Vec::new()})
    }
}

pub struct Data {
    name:       FileName,
    properties: Properties
}

impl Data {
    pub fn new(file_name: &str) -> Result<Data, Error> {
        Ok(Data{name: FileName::from_str(file_name)?, properties: Properties::default()})
    }
}

pub struct DirectoryName {
    name: DString<8>,
}

impl DirectoryName {
    pub fn from_str(dir_name: &str) -> Result<DirectoryName, Error> {
        let dir_name = dir_name.to_uppercase();
        Ok(DirectoryName{name: DString::from_str(dir_name.as_ref())?})
    }
}

pub struct FileName {
    name: DString<8>,
    ext:  Option<DString<3>>
}

impl FileName {
    pub fn from_str(file_name: &str) -> Result<FileName, Error> {
        let file_name = file_name.to_uppercase();
        let (name, ext) = {
            let mut sub_str = file_name.split('.');
            (Error::ok_or_new(sub_str.next(), || "File name can't be emplty".to_owned())?, sub_str.next())
        };

        Ok(FileName{name: DString::from_str(name)?, ext: {
            match ext {
                Some(ext) => Some(DString::from_str(ext)?),
                None      => None
            }
        }})
    }
}

pub struct Properties {
    lba:    Option<usize>,
    length: Option<usize>
}

impl Default for Properties {
    fn default() -> Self {
        Properties{lba: None, length: None}
    }
}
